unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ToolWin, ComCtrls, ExtCtrls, ImgList, Sockets,
  IdBaseComponent, IdComponent, IdUDPBase, IdUDPServer, IdSocketHandle,
  IdAntiFreezeBase, IdAntiFreeze, Contnrs;

type
  TfrmLogServer = class(TForm)
    Panel1: TPanel;
    edtHost: TEdit;
    lblHost: TLabel;
    edtPort: TEdit;
    lblPort: TLabel;
    tbrControl: TToolBar;
    Panel2: TPanel;
    btnExit: TToolButton;
    ToolButton2: TToolButton;
    btnPlay: TToolButton;
    mmoOutput: TMemo;
    ilstImages: TImageList;
    btnStop: TToolButton;
    LogServer: TIdUDPServer;
    AntiFreeze: TIdAntiFreeze;
    procedure btnPlayClick(Sender: TObject);
    procedure btnExitClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure LogServerUDPRead(Sender: TObject; AData: TStream;
      ABinding: TIdSocketHandle);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FCommunicationList: TObjectList;
    FStartPort: Integer;

    function CheckParameters: Boolean;

    procedure StartLogServer;
    procedure StopLogServer;
    procedure ClearCommunications;
  public
    { Public declarations }
  end;

var
  frmLogServer: TfrmLogServer;

implementation

uses
  LogServer;

{$R *.dfm}

procedure TfrmLogServer.btnPlayClick(Sender: TObject);
begin
    if (CheckParameters) then
    begin
        btnPlay.Enabled := False;
        btnExit.Enabled := False;
        btnStop.Enabled := True;

        StartLogServer;
    end; // if (CheckParameters) then
end; { TfrmLogServer.btnPlayClick }

procedure TfrmLogServer.btnExitClick(Sender: TObject);
begin
    Close;
end; { TfrmLogServer.btnExitClick }

procedure TfrmLogServer.btnStopClick(Sender: TObject);
begin
    StopLogServer;

    btnStop.Enabled := False;
    btnExit.Enabled := True;
    btnPlay.Enabled := True;
end; { TfrmLogServer.btnStopClick }

procedure TfrmLogServer.StartLogServer;
begin
    // Start the Log Server
    mmoOutput.Lines.Clear;
    mmoOutput.Lines.Add('Starting LogServer...');
    mmoOutput.Lines.Add('');
    mmoOutput.Lines.Add('Press the Stop button anytime to stop the LogServer.');
    mmoOutput.Lines.Add('');

    if not(LogServer.Active) then
        LogServer.Active := True;

    mmoOutput.Lines.Add('Awaiting connection...');
    mmoOutput.Lines.Add('');
end; { TfrmLogServer.StartLogServer }

procedure TfrmLogServer.StopLogServer;
begin
    mmoOutput.Lines.Add('Closing communication channels...');

    // Stop the communication channels
    ClearCommunications;

    if (LogServer.Active) then
        LogServer.Active := False;

    mmoOutput.Lines.Add('Done.')
end; { TfrmLogServer.StopLogServer }

function TfrmLogServer.CheckParameters: Boolean;
begin
    Result := True;
    if ((edtHost.Text = '') or (edtPort.Text = '')) then
    begin
        Result := False;
        MessageDlg('Host and/or Port information not supplied!', mtWarning, [mbOk], 0);
    end; // if
end; { TfrmLogServer.CheckParameters }

procedure TfrmLogServer.LogServerUDPRead(Sender: TObject; AData: TStream;
  ABinding: TIdSocketHandle);
var
    DataStringStream: TStringStream;
    Logger: TLogServer;
begin
    // On Receive of information
    DataStringStream := TStringStream.Create('');
    try
        DataStringStream.CopyFrom(AData, AData.Size);
        Logger := TLogServer.Create(mmoOutput);
        Logger.Initialize(DataStringStream.DataString, ABinding, edtHost.Text, StrToInt(edtPort.Text), FStartPort);
        FStartPort := FStartPort + 10;
        FCommunicationList.Add(Logger);
    finally
        DataStringStream.Free;
    end;
    Application.ProcessMessages;
end; { TfrmLogServer.LogServerUDPRead }

procedure TfrmLogServer.FormCreate(Sender: TObject);
begin
    FCommunicationList := TObjectList.Create;
    FStartPort := 7011;
end; { TfrmLogServer.FormCreate }

procedure TfrmLogServer.FormDestroy(Sender: TObject);
begin
    ClearCommunications;
    FCommunicationList.Free;
end; { TfrmLogServer.FormDestroy }

procedure TfrmLogServer.ClearCommunications;
var I: Integer;
    TempObject: TObject;
    TempLogServer: TLogServer;
begin
    for I := 0 to FCommunicationList.Count - 1 do
    begin
        TempObject := FCommunicationList[I];
        if (TempObject is TLogServer) then
        begin
            TempLogServer := (TempObject as TLogServer);
            TempLogServer.StopCommunication;
        end; // if TLogServer
    end; // while
    FCommunicationList.Clear;
end; { TfrmLogServer.ClearCommunication }

end.
